/********************************************************************************
*                                                                               *
*       AD1836A/ADSP-21161 SPI Code Register Initialization via SPI             *
*                                                                               *
*********************************************************************************/

/* ADSP-21161 System Register bit definitions */
#include 	"def21161.h"

.GLOBAL		Program_AD1836_regs_via_SPI;

// AD1836A codec SPI control/status register definitions
#define		READ_REG		0x0800
#define		WRITE_REG		0x0000
#define		DAC_CONTROL1	0x0000
#define		DAC_CONTROL2	0x1000
#define		DAC_VOLUME0		0x2000		
#define		DAC_VOLUME1		0x3000
#define		DAC_VOLUME2		0x4000
#define		DAC_VOLUME3		0x5000
#define		DAC_VOLUME4		0x6000
#define		DAC_VOLUME5		0x7000		
#define		ADC0_PEAK_LEVEL	0x8000
#define		ADC1_PEAK_LEVEL	0x9000
#define		ADC2_PEAK_LEVEL	0xA000
#define		ADC3_PEAK_LEVEL	0xB000
#define		ADC_CONTROL1	0xC000
#define		ADC_CONTROL2	0xD000
#define		ADC_CONTROL3	0xE000
#define		RESERVED_REG	0xF000

.section/dm dm_data;

/* Powerdown ADCs and DACs  */
.var powerdown_AD1836[4] = 	DAC_CONTROL1 | WRITE_REG | 0x004, 	
							DAC_CONTROL1 | WRITE_REG | 0x004,	// write twice, if you have the older rev. AD1836  	
							ADC_CONTROL1 | WRITE_REG | 0x080,
							ADC_CONTROL1 | WRITE_REG | 0x080;	// write twice, if you have the older rev. AD1836  	

// 	AD1836A codec register commands - Serial SPI 16-bit Word Format as follows:
// 			D15 to D12 	= Codec Register Address
//			D11 		= Read/Write register (1=rd, 0=wr)
//			D10 		= reserved bit, clear to zero
//			D9 to D0 	= Data Field for codec register

.var spi_tx_buf[16] = //program register commands 
					// addresses are "ORed", rd/wr, and register data for ease in reading register values 
					DAC_CONTROL1 | WRITE_REG | 0x002,	// I2S Mode, Interpolar mode 4x(96kHz)
					DAC_CONTROL2 | WRITE_REG | 0x000,	// 0 = all DACs are on
					DAC_VOLUME0  | WRITE_REG | 0x0FF,
					DAC_VOLUME1  | WRITE_REG | 0x0FF,
					DAC_VOLUME2  | WRITE_REG | 0x0FF,
					DAC_VOLUME3  | WRITE_REG | 0x0FF,
					DAC_VOLUME4  | WRITE_REG | 0x0FF,
					DAC_VOLUME5  | WRITE_REG | 0x0FF,
					ADC_CONTROL1 | WRITE_REG | 0x040,	// sample rate 96 kHz, 
					ADC_CONTROL1 | WRITE_REG | 0x040,
					ADC_CONTROL2 | WRITE_REG | 0x000,	// I2S Mode, Master, 24 Bits word width, all ADCs on
					ADC_CONTROL3 | WRITE_REG | 0x000,	// 256*Fs

					// read register commands - only the Peak level registers can be read
					ADC0_PEAK_LEVEL | READ_REG | 0x3ff, // status will be in rx_buf1a[13-19] memory locations	
					ADC1_PEAK_LEVEL | READ_REG | 0x3ff,
					ADC2_PEAK_LEVEL | READ_REG | 0x3ff,
					ADC3_PEAK_LEVEL | READ_REG | 0x000;
	
			
// read register
.var spi_rx_buf[16];
		

.section/pm pm_code;

/*********************************************************************************
*	                                                                             *      
*         			  Powerdown AD1836				                             *      	
*                                                                                *     
*********************************************************************************/

powerdown_reset_AD1836:

		ustat1 = dm(SYSCON);		// Clear Buffer Hang Disable in SYSCON
		bit clr ustat1 BHD;
		dm(SYSCON) = ustat1;
		nop;
		bit set mode1 CBUFEN;		// set circular buffer enable
		nop;
		bit set mode2 FLG0O;		// Flag 0 as output
		nop;
				
		r0=0x00000000;				// initially clear SPI control register
		dm(SPICTL)=r0;
		ustat1=dm(SPICTL);       						 
		nop;

		b0=powerdown_AD1836;		   
 		l0=1;					  	 
		m0=@powerdown_AD1836;	

		bit set ustat1 SPIEN|MS|PSSE|SPTINT|FLS0|BAUDR8|WL16|NSMLS|DF|CPHASE|CP|DCPH0;
	
		/* 
		SPIEN	SPI Port Enable
		MS	Master Select
		PSSE	Programmable Slave Select Enable
		SPTINT	SPITX Interrupt Enable
		FLS0	Flag select 0 
		BAUDR8	Baudrate 8
		WL16	Wordlength 16 bit
		SMLS	Non-Seamless Operation - delay before the next word transfer starts
		DF  	Data Format - MSB sent/received first
		CPHASE	Clock Phase - SPICLK starts toggling at the beginning of the first data transfer bit	
		CP	Clock Polarity - SPICLK high is the idle state
		DCPH0	Deselect SPIDS in CPHASE = 0
		*/

		dm(SPICTL) = ustat1;
		nop;

		// transmit
		lcntr=4, do (pc,2) until lce;
		r3=dm(i0,m0);
		dm(SPITX)=R3;
		nop;
		nop;

Wait_Approx_1s:
		lcntr = 30000, do waitloop until lce;	// wait after powerdown, workaround for anomaly
				nop;
				nop;
				nop;
waitloop:		nop;


/********************************************************************************
*	                                                                            *
*           Program SPI Control Register                                        *	
*																				*
*********************************************************************************/

Program_AD1836_regs_via_SPI:

		ustat1 = dm(SYSCON);		// Clear Buffer Hang Disable in SYSCON
		bit clr ustat1 BHD;
		dm(SYSCON) = ustat1;
		nop;
		bit set mode1 CBUFEN;		// set circular buffer enable
		nop;
		bit set mode2 FLG0O;		// Flag 0 as output
		nop;
				
		r0=0x00000000;			 	// initially clear SPI control register
		dm(SPICTL)=r0;
		ustat1=dm(SPICTL);       						 
		nop;
		
		b0=spi_tx_buf;				// get address of SPI transmit data array, i0=b0
		l0=@spi_tx_buf;				// get length of SPI transmit data array
		m0=1;						// set modifier to 1						 
		
		b8=spi_rx_buf;
		l8=@spi_rx_buf;
		m8=1;
		
		i7=0x50000;
		m7=1;

		bit set ustat1 SPIEN|MS|PSSE|SPTINT|FLS0|BAUDR8|WL16|NSMLS|DF|CPHASE|CP|DCPH0;
		
		/* 
		SPIEN	SPI Port Enable
		MS		Master Select
		PSSE	Programmable Slave Select Enable
		SPTINT	SPITX Interrupt Enable
		FLS0	Flag select 0 
		BAUDR8	Baudrate 8
		WL16	Wordlength 16 bit
		SMLS	Seamless Operation - delay before the next word transfer starts
		DF  	Data Format - MSB sent/received first
		CPHASE	Clock Phase - SPICLK starts toggling at the beginning of the first data transfer bit	
		CP		Clock Polarity - SPICLK high is the idle state
		DCPH0	Deselect SPIDS in CPHASE = 0
		*/

		dm(SPICTL) = ustat1;
		nop;
		
		// transmit
		lcntr=15, do (pc,2) until lce;
		r3=dm(i0,m0);
		dm(SPITX)=R3;
		nop;
		nop;

		R4=dm(SPIRX);

		// receive
		lcntr=15, do (pc,2) until lce;
		dm(i7,m7)=R4;
		nop;
		nop;
			
	rts;

