/********************************************************************************
*																				*       
*			AD1836/ADSP-21161 SPORT Initialization Driver                       *
*																				*
*********************************************************************************/
	
/* ADSP-21161 System Register bit definitions */
#include 	"def21161.h"

.GLOBAL		Program_SPORT13_I2S_Registers;
.GLOBAL		Program_SPORT13_DMA_Channels;
.GLOBAL		AD1836_Codec_Initialization;
.GLOBAL		rx1a_buf, tx3a_buf, tx3b_buf;
.GLOBAL		Wait_Approx_999us;

#define PCI 0x40000		// bit 18, PCI = program controlled interrupt
#define N 1024

.section /dm    dm_codec;

/* TCB = "Transfer Control Block" */
/* TCB format for SPORT:
				  GPx ("general purpose"),
				  CPx ("Chain Point register"; points to last address (IIx) of
			   								   next TCB to jump to
				                               upon completion of this TCB.),
				  Cx  (length of source buffer),
				  IMx (source buffer step size),
				  IIx (source buffer index (initialized to start address))       */

.var 	sp1a_tcb[5] = 0, sp1a_tcb+4, N, 1, rx1a_buf;  /* SPORT1 receive channel a tcb */
.var 	sp3a_tcb[5] = 0, sp3a_tcb+4, N, 1, tx3a_buf;  /* SPORT3 transmit channel a tcb */
.var 	sp3b_tcb[5] = 0, sp3b_tcb+4, N, 1, tx3b_buf;  /* SPORT3 transmit channel b tcb */

.section /pm 	pm_code;

/********************************************************************************
*																				*
* 		    DMA Controller Programming For SPORT1 and SPORT3                    *
*																				*
*********************************************************************************/

Program_SPORT13_DMA_Channels:

/* PCI assignments  */

	/* SPORT1 dma control rx setup channel A */
	ustat1 = dm(sp1a_tcb + 1);		/* get DMA chaining internal mem pointer containing rx_buf address */
	bit set ustat1 PCI;				/* set the pci bit */	
	dm(sp1a_tcb + 1) = ustat1;    	/* write DMA receive block chain pointer to TCB buffer*/
				                 	/* receive block chain pointer, initiate rx1 DMA transfers */

	/* SPORT3 dma control tx setup channel A  */
	ustat1 = dm(sp3a_tcb + 1);		/* get DMA chaining internal mem pointer containing tx_buf address */
	bit set ustat1 PCI;				/* set the pci bit */
	dm(sp3a_tcb + 1) = ustat1; 		/* write DMA transmit block chain pointer to TCB buffer */
				                 	/* transmit block chain pointer, initiate tx3 DMA transfers */

	/* SPORT3 dma control tx setup channel B  */
	ustat1 = dm(sp3b_tcb + 1);		/* get DMA chaining internal mem pointer containing tx_buf address */
	bit set ustat1 PCI;				/* set the pci bit */
	dm(sp3b_tcb + 1) = ustat1;  	/* write DMA transmit block chain pointer to TCB buffer */
				                 	/* transmit block chain pointer, initiate tx3 DMA transfers */
rts;

/********************************************************************************
*																				*
* 		    SPORT1 and SPORT3 Register setup				                    *
*																				*
*********************************************************************************/

Program_SPORT13_I2S_Registers:

	// SPORT1 control register set up as a receiver in I2S
	R0 = SPEN_A|SLEN32|OPMODE|LFIRST|SDEN_A|SCHEN_A;
	dm(SPCTL1) = R0;			/* sport 1 control register SPCTL1 = 0x000D09F1 */

/*				SPEN_A		SPORT Enable A
				SLEN32		Serial Word Length - 1 
				MSTR0		external clock and word-select, RX is slave
				OPMODE		Operation Mode: I2S
				LFIRST		start left data first
				SDEN_A		SPORT transmit DMA enable A channel
				SCHEN_A		DMA Chaining Enable A channel
				DDIR0		Data Direction Control: Enable Active Receive Buffers RX1A/RX1B
*/

	// SPORT3 control register set up as a transmitter in I2S
	R0 = SPEN_A|SLEN32|OPMODE|DITFS|LFIRST|SDEN_A|SCHEN_A|SDEN_B|SCHEN_B|FS_BOTH|SPEN_B|DDIR;
	dm(SPCTL3) = R0;			/* sport 3 control register, SPCTL3 = 0x273D09F1 */
	
/*				SPEN_A		SPORT Enable A
				SLEN32		Serial Word Length - 1 
				MSTR0		external clock and word-select, TX is slave
				OPMODE		Operation Mode: I2S 
				DITS		Data independent tx Framesync (if DDIR=1)
				LFIRST		start left data first
				SDEN_A		SPORT transmit DMA enable A channel
				SCHEN_A		DMA Chaining Enable A channel
				SDEN_B		SPORT transmit DMA enable B channel
				SCHEN_B		DMA Chaining Enable B channel
				FS_BOTH		Frame Sync both enable; issue WS only if data is present in both TX
				SPEN_B		SPORT Enable B 
				DDIR		Data Direction Control: Enable Active Transmit Buffers TX3A/TX3B
*/


/* Kick off DMA chaining of SPORT1&3 */

r8=sp1a_tcb+4; 
r9=sp3a_tcb+4;
r10=sp3b_tcb+4;

dm(CP1A)=r8;			// SPORT1 channel A DMA chain enable
dm(CP3A)=r9;			// SPORT3 channel A DMA chain enable
dm(CP3B)=r10;			// SPORT3 channel B DMA chain enable



	RTS;

 

